/*
modified and reduced from
https://github.com/quarto-dev/quarto-cli/tree/main/src/resources/formats/revealjs/plugins/line-highlight
*/

/* for source-line-numbers */
body:not(.modal-open) div.sourceCode pre code.has-line-highlights > span:not(.highlight-line) {
  opacity: 0.4;
}

/* for output-line-numbers */
body:not(.modal-open) div.sourceCode pre.highlight code.has-line-highlights > span:not(.highlight-line) {
  opacity: 0.4;
}

/* Adding hovering effect */
/* from https://github.com/shafayetShafee/line-highlight/issues/1#issue-1639343001 by @GShotwell*/

body:not(.modal-open) div.sourceCode:hover pre code.has-line-highlights > span:not(.highlight-line) {
  opacity: 0.4;
}


body:not(.modal-open) div.sourceCode:hover pre code.has-line-highlights > span.highlight-line {
  font-weight: 600;
}


body:not(.modal-open) div.sourceCode:hover pre.highlight code.has-line-highlights > span:not(.highlight-line) {
  opacity: 0.4;
}

body:not(.modal-open) div.sourceCode:hover pre.highlight code.has-line-highlights > span.highlight-line {
  font-weight: 600;
}