# Import all patches to ensure they are applied
from . import (
    barplot,
    boxplot,
    candlestick,
    clear,
    heatmap,
    highlight,
    histogram,
    lineplot,
    scatterplot,
    regplot,
    kdeplot,
    mplfinance,
)
