# Keywords used to detect smooth/regression lines in MAIDR
SMOOTH_KEYWORDS = [
    "smooth",
    "lowess",
    "loess",
    "regression",
    "linear regression",
    "linear fit",
    "fit",
    "kde",
    "density",
    "gaussian",
]
